@@ -37,7 +37,6 @@ abstract class rcube_session
     protected $vars;
     protected $now;
     protected $lifetime;
-    protected $time_diff    = 0;
     protected $reloaded     = false;
     protected $appends      = [];
     protected $unsets       = [];
@@ -52,7 +51,7 @@ abstract class rcube_session
     /**
      * Blocks session data from being written to database.
      * Can be used if write-race conditions are to be expected
-     * @var boolean
+     * @var bool
      */
     public $nowrite = false;
 
@@ -266,7 +265,7 @@ abstract class rcube_session
     /**
      * Register additional garbage collector functions
      *
-     * @param mixed Callback function
+     * @param mixed $func Callback function
      */
     public function register_gc_handler($func)
     {
@@ -738,7 +737,7 @@ abstract class rcube_session
     /**
      * Writes debug information to the log
      *
-     * @param string Log line
+     * @param string $line Log line
      */
     function log($line)
     {
